#!/usr/bin/python2.5

from __future__ import with_statement

from os import system, makedirs

from unittest import TestCase, main
from cq2utils import CallTrace, autostart, getRequest, postRequest, wheelOfTime
from os.path import join, isdir, isfile
from os import remove
from time import sleep, time
from amara.binderytools import bind_file
from meresco.components.http import ObservableHttpServer
from meresco.framework import be
from urllib import urlencode
from lxml.etree import tostring

from bzv.server import main as serverMain
from weightless import Reactor, compose
from socket import socket
from random import randint

global port
global reactor
port = None
reactor = None
integrationTempdir = '/tmp/bzv/integrationtest'
inboxDirectory = join(integrationTempdir, 'inboxDirectory')
doneDirectory = join(integrationTempdir, 'doneDirectory')

class IntegrationTest(TestCase):
    
    def testExplain(self):
        header, body = getRequest(reactor, port, '/sru', {}, parse=False)
        self.assertTrue('explainResponse' in body, body)
        self.assertTrue('port>%s</' % port in body, body)

    
    def testHtml(self):
        header, response = getRequest(reactor, port, '/elsevier', {}, parse=False)
        self.assertTrue('</html>' in response, response)

    def testSruQuery(self):
        self.assertSruQuery(79, 'meresco.exists=true')
        self.assertSruQuery(19, 'meta.repository.id exact neeo')
        self.assertSruQuery(60, 'meta.repository.id exact elsevieroai')
        self.assertSruQuery(19, 'neeo')

    def testModsRecordSchemaIsDefault(self):
        result = self._doQuery({'query':'meresco.exists=true'})
        self.assertEquals('mods', str(result.searchRetrieveResponse.records.record.recordSchema))
        xmlOutput = result.searchRetrieveResponse.records.record.recordData.xml()
        self.assertTrue('<mods xmlns="http://www.loc.gov/mods/v3"' in xmlOutput, xmlOutput)

    def testDrilldown(self):
        #result = self.doDrilldown('meresco.exists=true', 'mods.name.namePart')
        result = self.doDrilldown('meresco.exists=true', 'mods.relatedItem.titleInfo.title')
        navigator = result.extraResponseData.drilldown.term_drilldown.navigator
        self.assertEquals(1, len(navigator))
        #self.assertEquals('mods.name.namePart', str(navigator.name))
        self.assertEquals('mods.relatedItem.titleInfo.title', str(navigator.name))
        self.assertEquals(10, len(navigator.item))
 
        result = self.doDrilldown('meresco.exists=true', 'meta.repository.id')
        navigator = result.extraResponseData.drilldown.term_drilldown.navigator
        self.assertEquals(1, len(navigator))
        self.assertEquals('meta.repository.id', str(navigator.name))
        self.assertEquals(2, len(navigator.item))
        self.assertEquals(set(['elsevieroai', 'neeo']), set([str(item) for item in navigator.item]))

        #result = self.doDrilldown('meresco.exists=true', 'mods.name.type')
        #navigator = result.extraResponseData.drilldown.term_drilldown.navigator

        #self.assertEquals(5, len(navigator.item))
        #self.assertEquals(set(['code', 'corporate', 'family', 'given', 'personal']), set([str(item) for item in navigator.item]))
        
    def testMultiLevelDrilldownInHtml(self):
        html = self.getPage('/?query=')
        self.assertFalse('&drilldown=drilldown.norm.date.decennium' in html, html[html.index('&drilldown='):])
        self.assertTrue('&drilldown=drilldown.norm.date.year' in html, html[html.index('&drilldown='):])

    def testNormalizationEmptyDateInXml(self):
        result = self._doQuery({'query': 'meta.upload.id exact "elsevieroai:elsevier:CXT0242A:01988859:00690004:08000608"', 'recordSchema': 'extra'})
        extraXml = result.searchRetrieveResponse.records.record.recordData.extra
        self.assertEquals('', str(extraXml.norm.date))

    def testRss(self):
        headers, body = getRequest(reactor, port, '/rss', {'query': 'meta=neeo'}, parse='lxml')
        items = body.xpath('/rss/channel/item')
        self.assertEquals(15, len(items))
        self.assertEquals(['IntegrationTest BZV'], body.xpath('/rss/channel/title/text()'))

    def testUndatedRecordsGetUnknown(self):
        self.assertSruQuery(6, 'mods.originInfo.dateIssued=unknown')
        
    def testRateRecord(self):
        ratingFile = join(integrationTempdir, 'storage', 'elsevieroai', 'elsevier:CXT0242A:01988859:00690004:08000608', 'rating')
        isfile(ratingFile) and remove(ratingFile)
        recordId = 'elsevieroai:elsevier:CXT0242A:01988859:00690004:08000608'

        result = self._doQuery({'query': 'meta.upload.id exact "%s"' % recordId, 'recordSchema': 'rating'})
        ratingXml = result.searchRetrieveResponse.records.record.recordData.rating
        self.assertEquals('', ratingXml)

        header,body = postRequest(reactor, port, '/rateRecord', urlencode({'recordId': recordId, 'value': 3}), parse=False)
        self.assertTrue('302 ' in header, header+body)

        result = self._doQuery({'query': 'meta.upload.id exact "%s"' % recordId, 'recordSchema': 'rating'})
        ratingXml = result.searchRetrieveResponse.records.record.recordData.rating
        self.assertEquals('3000', str(ratingXml.averageScore))
        
    def testDidlIsStored(self):
        result = self.assertSruQuery(19, 'meta=neeo', recordSchema='didl')
        xmlData = result.xml()
        self.assertFalse("'didl')' does not exist." in xmlData, xmlData)
        self.assertEquals('urn:nbn:nl:ui:12-402853', str(result.rootNode.searchRetrieveResponse.records.record.recordData.DIDL.Item.Descriptor.Statement.Identifier))
        
    def doDrilldown(self, query, drilldownField):
        message = self._doQuery({'query':query, 'x-term-drilldown': drilldownField})
        result = message.searchRetrieveResponse
        return result

    def assertSruQuery(self, numberOfRecords, query, **kwargs):
        queryParams = {'query': query}
        queryParams.update(kwargs)
        message = self._doQuery(queryParams)
        result = message.searchRetrieveResponse
        self.assertEquals(numberOfRecords, int(str(result.numberOfRecords)))
        return result

    def _doQuery(self, arguments):
        queryArguments = {'version': '1.1', 'operation': 'searchRetrieve'}
        queryArguments.update(arguments)
        header, body = getRequest(reactor, port, '/sru', queryArguments)
        return body
    
    def getPage(self, url, parse=False):
        header, body = getRequest(reactor, port, url, {}, parse=parse)
        return body

def createDatabase():
    allOutput = ""
    recordPacking = 'xml'
    start = time()
    print "Creating database in", integrationTempdir
    sourceFiles = glob('sahara_output/*.updateRequest')
    for updateRequestFile in sorted(sourceFiles):
        print 'Sending:', updateRequestFile
        header, body = postRequest(reactor, port, '/update', open(updateRequestFile).read())
        if '200 Ok' not in header:
            print 'No 200 Ok response, but:'
            print header
            exit(123)
        if "srw:diagnostics" in body.xml():
            print body.xml()
            exit(1234)
    print "Finished creating database in %s seconds" % (time() - start)
    print "Giving the server index some time for a auto refresh"
    wheelOfTime(reactor, 15)
    print "Finished creating database"

if __name__ == '__main__':
    from sys import argv
    if not '--fast' in argv:
        system('rm -rf ' + integrationTempdir)
        makedirs(inboxDirectory)
        makedirs(doneDirectory)
    port = randint(50000,60000)
    reactor = Reactor()
    observableHttpServer = ObservableHttpServer(reactor, port)
    sruUpdateServer = observableHttpServer
    server = be(serverMain(reactor,
        observableHttpServer,
        sruUpdateServer,
        host='localhost',
        portNumber=port,
        databasePath=integrationTempdir,
        configDict={
            'defaultSortKeys':'sort.mods.originInfo.dateIssued,,1',
            'applicationName':'IntegrationTest BZV',
########### note the following paths are installation dependent
            'dynamicPath'    :'../html/dynamic',
            'staticPath'     :'../html/static'
        }))
    #server.printTree()
    server.once.observer_init()

    if '--fast' in argv and isdir(integrationTempdir):
        argv.remove('--fast')
        print "Reusing database in", integrationTempdir
    else:
        createDatabase()
    main()
